@@ -148,20 +148,20 @@ IDE <- function(f, data, dt, process_basis = NULL, kernel_basis = NULL, grid_siz
 
   update_alpha <- function() {
     alphahat <<-  cholsolve(Q = Qpost,
-                                  y = t(PHI_obs) %*% Q_eps %*% (Z - X_obs %*% betahat),
-                                  perm = TRUE,
-                                  cholQp = Qpostchol$Qpermchol,
-                                  P = Qpostchol$P)
+                            y = t(PHI_obs) %*% Q_eps %*% (Z - X_obs %*% betahat),
+                            perm = TRUE,
+                            cholQp = Qpostchol$Qpermchol,
+                            P = Qpostchol$P)
   }
 
   #update_beta <- function(X_obs, PHI_obs, Q_eps, Qpost_cholsolve, Z) {
   update_betahat <- function() {
     Qpost_cholsolve <- function(y) {
       cholsolve(Q = Qpost,
-                      y = y,
-                      perm = TRUE,
-                      cholQp = Qpostchol$Qpermchol,
-                      P = Qpostchol$P)
+                y = y,
+                perm = TRUE,
+                cholQp = Qpostchol$Qpermchol,
+                P = Qpostchol$P)
     }
 
     tPHIQepsZ <- t(PHI_obs) %*% Q_eps %*% Z
@@ -369,16 +369,16 @@ predict.IDE <- function(object, newdata = NULL, covariances = FALSE, ...) {
   Qpost_dense <- densify(Qpost,t(PHI_pred) %*% PHI_pred)
   Qpostchol <- cholPermute(Qpost_dense)
   Ssparseinv <- Takahashi_Davis(Q = Qpost_dense,
-                                      cholQp = Qpostchol$Qpermchol,
-                                      P = Qpostchol$P)
+                                cholQp = Qpostchol$Qpermchol,
+                                P = Qpostchol$P)
   newdata$Ypredse <- rowSums(PHI_pred * (PHI_pred %*% Ssparseinv)) %>%
     as.numeric() %>% sqrt()
   if(covariances == TRUE) {
     if(nrow(PHI_pred) > 4000)
       stop("Cannot generate covariances for more than 4000 locations")
-    S <- cholsolveAQinvAT(PHI_pred,
-                                Lp = Qpostchol$Qpermchol,
-                                P = Qpostchol$P)
+    S <- cholsolveAQinvAT(A = PHI_pred,
+                          Lp = Qpostchol$Qpermchol,
+                          P = Qpostchol$P)
     newdata <- list(newdata = newdata,
                     Cov = S)
   }
@@ -447,7 +447,7 @@ show_kernel <- function(IDEmodel, scale = 1) {
       s$s_grid_df$ver <-  eval_basis(kernel_basis[[4]],s$s_grid_mat) %*% k[[4]] %>% as.numeric()
       ggplot(data=s$s_grid_df, aes(x=s1, y=s2)) +
         geom_segment(aes(xend=s1-hor*scale, yend=s2-ver*scale),
-                         colour = "black", size = 0.2,
+                     colour = "black", size = 0.2,
                      arrow = arrow(length = unit(0.1,"cm"))) +
         theme_bw()
     } else {
@@ -478,6 +478,7 @@ constant_basis <- function() {
 #' @param T number of time points to simulate
 #' @param nobs number of observations randomly scattered in the domain and fixed for all time intervals
 #' @param k_spat_invariant flag indicating whether to simulate using a spatially-invariant kernel or a spatially-variant one
+#' @param IDEmodel object of class IDE to simulate form (optional)
 #' @details The domain considered is [0,1] x [0,1], and an IDE is simulated on top of a fixed effect comprising of an intercept, a linear horizontal effect, and a linear vertical effect (all with coefficients 0.2). The measurement-error variance and the variance of the additive disturbance are both 0.0001. When a spatially-invariant kernel is used, the following parameters are fixed: \eqn{\theta_{p,1} = 150}, \eqn{\theta_{p,2} = 0.002}, \eqn{\theta_{p,3} = -0.1}, and \eqn{\theta_{p,4} = 0.1}. See \code{\link{IDE}} for details on these parameters. When a spatially-varying kernel is used, \eqn{\theta_{p,1} = 200}, \eqn{\theta_{p,2} = 0.002}, and \eqn{\theta_{p,3}(s), \theta_{p,4}(s)} are smooth spatial functions simulated on the domain.
 #'
 #' @return A list containing the simulated process in \code{s_df}, the simulated data in \code{z_df}, the data as \code{STIDF} in \code{z_STIDF}, plots of the process and the observations in \code{g_truth} and \code{g_obs}, and the IDE model used to simulate the process and data in \code{IDEmodel}.
@@ -487,102 +488,152 @@ constant_basis <- function() {
 #' SIM1 <- simIDE(T = 5, nobs = 100, k_spat_invariant = 1)
 #' SIM2 <- simIDE(T = 5, nobs = 100, k_spat_invariant = 0)
 
-simIDE <- function(T = 9, nobs = 100, k_spat_invariant = 1) {
+simIDE <- function(T = 9, nobs = 100, k_spat_invariant = 1, IDEmodel = NULL) {
   ## Suppress bindings warning
-  val <- s1 <- s2 <- z <- NULL
+  timeind <- val <- s1 <- s2 <- z <- NULL
 
-  set.seed(1)
-  zlocs <- data.frame(s1 = runif(100),
-                      s2 = runif(100))
+  if(is.null(IDEmodel)) {
 
-  ## Spatial decomposition
-  Y_basis <- auto_basis(manifold = plane(),
-                        data = SpatialPoints(zlocs),
-                        regular = 1,
-                        nres = 2)
-  r <- nbasis(Y_basis)
+    set.seed(1)
+    zlocs <- data.frame(s1 = runif(100),
+                        s2 = runif(100))
 
-  ## Kernel decomposition
-  G_const <- constant_basis()
+    ## Spatial decomposition
+    Y_basis <- auto_basis(manifold = plane(),
+                          data = SpatialPoints(zlocs),
+                          regular = 1,
+                          nres = 2)
+    r <- nbasis(Y_basis)
 
-  ## Regression coeffocients
-  beta <- c(0.2,0.2,0.2)
+    ## Kernel decomposition
+    G_const <- constant_basis()
 
-  ## Other parameters
-  sigma2_eta <- 0.01^2
-  sigma2_eps <- 0.01^2
+    ## Regression coeffocients
+    beta <- c(0.2,0.2,0.2)
+
+    ## Other parameters
+    sigma2_eta <- 0.01^2
+    sigma2_eps <- 0.01^2
+
+    ## Spatial domain
+    bbox <- matrix(c(0,0,1,1),2,2)
+    s <- construct_grid(bbox, 41)
+    alpha <- matrix(0,r,T)
+
+    ## Kernel
+    if(k_spat_invariant) {
+      K_basis <- list(G_const, G_const, G_const, G_const)
+      k <- list(150, 0.002, -0.1, 0.1)
+      alpha[65,1] <- 1
+    } else {
+      G <- auto_basis(plane(), data = SpatialPoints(s$s_grid_df),nres = 1)
+      nbk <- nbasis(G)
+      K_basis <- list(G_const, G_const, G, G)
+      k <- list(200, 0.002, 0.1*rnorm(nbk), 0.1*rnorm(nbk))
+      alpha[sample(1:r,10),1] <- 1
+    }
+    time_map <- data.frame(timeind = paste0("Y",0:(T-1)),
+                           time = as.Date(0:(T-1), origin = "2017-12-01"),
+                           stringsAsFactors = FALSE)
+  } else {
+    Y_basis <- IDEmodel$get("process_basis")
+    r <- nbasis(Y_basis)
+    beta <- coef(IDEmodel)
+    sigma2_eta <- c(IDEmodel$get("sigma2_eta"))
+    sigma2_eps <- c(IDEmodel$get("sigma2_eps"))
+    s <- IDEmodel$get("s")
+    T <- IDEmodel$get("T")
+    nobs <- nrow(IDEmodel$get("data"))
+    K_basis <- IDEmodel$get("kernel_basis")
+    k <- IDEmodel$get("k")
+    alpha <- matrix(0,r,T)
+    alpha[,1] <- sqrt(sigma2_eta) * rnorm(r)
+    time_map <- data.frame(timeind = paste0("Y",0:(T-1)),
+                           time = IDEmodel$get("time_points"),
+                           stringsAsFactors = FALSE)
+  }
+
+  ## Construct matrices
   Sigma_eta <- sigma2_eta * Diagonal(r)
   Sigma_eps <- sigma2_eps * Diagonal(nobs * T)
   Q_eta <- Sigma_eta %>% solve()
   Q_eps <- Sigma_eps %>% solve()
 
-  ## Simulate process
-  bbox <- matrix(c(0,0,1,1),2,2)
-  s <- construct_grid(bbox, 41)
   Mfun <- construct_M(Y_basis, s)
-  alpha <- matrix(0,nbasis(Y_basis),T)
-  if(k_spat_invariant) {
-    K_basis <- list(G_const, G_const, G_const, G_const)
-    k <- list(150, 0.002, -0.1, 0.1)
-    alpha[65,1] <- 1
-  } else {
-    G <- auto_basis(plane(), data = SpatialPoints(s$s_grid_df),nres = 1)
-    nbk <- nbasis(G)
-    K_basis <- list(G_const, G_const, G, G)
-    k <- list(200, 0.002, 0.1*rnorm(nbk), 0.1*rnorm(nbk))
-    alpha[sample(1:r,10),1] <- 1
-  }
   M <- Mfun(K_basis, k)
   PHI <- eval_basis(Y_basis, s$s_grid_mat)
   s$s_grid_df$Y0 <- (PHI %*% alpha[,1]) %>% as.numeric()
   for(i in 1:(T-1)) {
-    alpha[,i+1] <- (M %*% alpha[,i]) %>% as.numeric() + sqrt(sigma2_eta)*rnorm(nbasis(Y_basis))
+    alpha[,i+1] <- (M %*% alpha[,i]) %>% as.numeric() +
+      sqrt(sigma2_eta)*rnorm(r)
     s$s_grid_df[paste0("Y",i)] <- (PHI %*% alpha[,i+1]) %>% as.numeric()
   }
 
-  time_map <- data.frame(time = paste0("Y",0:(T-1)),
-                         t = as.Date(0:(T-1), origin = "2017-12-01"),
-                         stringsAsFactors = FALSE)
-  s_long <- gather(s$s_grid_df, time, val, -s1, -s2) %>%
-    left_join(time_map, by = "time") %>%
-    select(-time)
+  s_long <- gather(s$s_grid_df, timeind, val, -s1, -s2) %>%
+    left_join(time_map, by = "timeind") %>%
+    select(-timeind)
+  if(is.null(IDEmodel)) X_proc <-  cbind(1, s_long[,c("s1","s2")]) %>% as.matrix()
+
+
+  if(is.null(IDEmodel)) {
+    fixed_effects <- (X_proc %*% beta) %>% as.numeric()
+    s_long$val <- s_long$val + fixed_effects## Observe process
+    zlocs <- data.frame(s1 = runif(nobs),
+                        s2 = runif(nobs))
+    PHI_obs_1 <- eval_basis(Y_basis, zlocs[,1:2] %>% as.matrix())
+    PHI_obs <- do.call("bdiag", lapply(1:T, function(x) PHI_obs_1))
+    X_obs <-  cbind(1, do.call("rbind", lapply(1:T, function(x) zlocs))) %>% as.matrix()
+    Z <- X_obs %*% beta + PHI_obs %*% c(alpha) +
+      sqrt(sigma2_eps) * rnorm(nrow(PHI_obs))
+    z_df <- data.frame(expand.grid.df(zlocs, data.frame(time = time_map$time)))
+    z_df$z <- Z %>% as.numeric()
+  } else {
+    fixed_effects <- 0
+    s_long$val <- s_long$val + fixed_effects
+    PHI_obs <- IDEmodel$get("PHI_obs")
+    X_obs <- IDEmodel$get("X_obs")
+    Z <- X_obs %*% beta + PHI_obs %*% c(alpha) +
+      sqrt(sigma2_eps) * rnorm(nrow(PHI_obs))
+    z_df <- as.data.frame(IDEmodel$get("data"))
+    z_df[[all.vars(IDEmodel$get("f"))[1]]] <- Z %>% as.numeric()
+  }
 
-  X_proc <-  cbind(1, s_long[,c("s1","s2")]) %>% as.matrix()
-  s_long$val <- s_long$val + (X_proc %*% beta %>% as.numeric())
 
-  ## Observe process
-  zlocs <- data.frame(s1 = runif(nobs),
-                      s2 = runif(nobs))
-  PHI_obs_1 <- eval_basis(Y_basis, zlocs[,1:2] %>% as.matrix())
-  PHI_obs <- do.call("bdiag", lapply(1:T, function(x) PHI_obs_1))
-  X_obs <-  cbind(1, do.call("rbind", lapply(1:T, function(x) zlocs))) %>% as.matrix()
-  Z <- X_obs %*% beta + PHI_obs %*% c(alpha) + sqrt(sigma2_eps) * rnorm(nrow(PHI_obs))
-  z_df <- data.frame(expand.grid.df(zlocs, data.frame(t = time_map$t)))
-  z_df$z <- Z %>% as.numeric()
   g_obs <- ggplot(z_df) + geom_point(aes(s1, s2, colour = z)) +
-    facet_wrap(~t) +
-    scale_colour_distiller(palette = "Spectral") +
-    coord_fixed(xlim=c(0,1), ylim = c(0,1))
+    facet_wrap(~time) +
+    scale_colour_distiller(palette = "Spectral")
+  if(is.null(IDEmodel)) g_obs <- g_obs + coord_fixed(xlim=c(0,1), ylim = c(0,1))
 
-  g_truth <- ggplot(s_long) + geom_tile(aes(s1,s2,fill=val)) + facet_wrap(~t) +
+  g_truth <- ggplot(s_long) + geom_tile(aes(s1,s2,fill=val)) +
+    facet_wrap(~time) +
     scale_fill_distiller(palette="Spectral",
-                         limits = c(min(c(z_df$z,s_long$val)),max(z_df$z,s_long$val))) +
-    coord_fixed(xlim=c(0,1), ylim = c(0,1))
+                         limits = c(min(c(z_df$z,s_long$val)),
+                                    max(z_df$z,s_long$val)))
+  if(is.null(IDEmodel)) g_truth <- g_truth + coord_fixed(xlim=c(0,1), ylim = c(0,1))
 
   ## Data as STIDF
-  z_STIDF <- STIDF(sp = SpatialPoints(z_df[,c("s1","s2")]),
-                   time = z_df$t,
-                   data = z_df)
+  if(is.null(IDEmodel)) {
+    cnames <- c("s1","s2")
+    z_STIDF <- STIDF(sp = SpatialPoints(z_df[,cnames]),
+                     time = z_df$time,
+                     data = select(z_df, -time, -s1, -s2))
+  } else {
+    z_STIDF <- IDEmodel$get("data")
+    z_STIDF$z <- as.numeric(Z)
+  }
+
 
   ## IDEmode used to generate data
-  IDEmodel <- IDE(f = z ~ s1 + s2 + 1,
-                  data = z_STIDF,
-                  dt = as.difftime(1, units = "days"),
-                  grid_size = 41,
-                  kernel_basis = K_basis)
-  IDEmodel$set(sigma2_eps = sigma2_eps,
-               sigma2_eta = sigma2_eta,
-               k = k)
+  if(is.null(IDEmodel)) {
+    IDEmodel <- IDE(f = z ~ s1 + s2 + 1,
+                    data = z_STIDF,
+                    dt = as.difftime(1, units = "days"),
+                    grid_size = 41,
+                    kernel_basis = K_basis)
+    IDEmodel$set(sigma2_eps = sigma2_eps,
+                 sigma2_eta = sigma2_eta,
+                 k = k)
+  }
 
   list(s_df = s_long,
        z_df = z_df,
